#include "internls\evrythng.h"
#pragma hdrstop

// am9513a.cpp
// (C) 1993, 1995 by Heiko Damerow MPG AG "Rntgenbeugung"

#include "detecuse\am9513a.h"

//******** Methoden fr den Zhler-IC Am9513A **********************************
TAm9513a::TAm9513a( void )
{
  bIsStopped = FALSE;
  bIsLatched = FALSE;
  wLowTicks = 1000;
  wHighTicks = 100;
  wLowCounts = 1000;
  wHighCounts = 100;
  nbChipId = 0;
};

void TAm9513a::LoockUp( LPCSTR Section )
{
  char buf[ MaxString ];

  GetPrivateProfileString( Section,"IOAddr","0x230",buf,MaxString,GetHWFile( ) ); 
  sscanf( buf,"%x",&nBaseAddr );
  GetPrivateProfileString( Section,"TimeCorrection","1.0",buf,MaxString,GetHWFile( ) ); 
  sscanf( buf,"%f",&fTimeCorrection );
};

int TAm9513a::Init( void )
{
  WORD wValue;

  // Kommunikations-Test
  ChooseDataPtr( CNT1,CMR );
  WriteData( 0xA121 );
  ChooseDataPtr( CNT1,CMR );
  wValue = ReadData( );
  if( wValue != 0xA121 )
    return FALSE;
  // Zhler zurcksetzen
  WriteCmd( 0xFF );
  WriteCmd( 0xF9 );
  // Konfigurieren des Mastermode-Register
  ChooseDataPtr( CTRG,MMR );
  wValue = 0xCA00;
  WriteData( wValue ); // F1 --> FOUT
  // Konfigurieren des TimeTick-Zhler
  ChooseDataPtr( CNT1,CMR );
  wValue = 0xA121;
  WriteData( wValue );
  ChooseDataPtr( CNT2,CMR );
  wValue = 0x0202;
  WriteData( wValue );
  // Konfigurieren des Impuls-Zhler
  ChooseDataPtr( CNT3,CMR );
  wValue = 0xA321;
  WriteData( wValue );
  ChooseDataPtr( CNT4,CMR );
  wValue = 0x0402;
  WriteData( wValue );
  // Konfigurieren des Akustik-Zhler
  ChooseDataPtr( CNT5,CMR );
  wValue = 0x0C21;
  WriteData( wValue );
  return TRUE;
};

int TAm9513a::IOCTL( TIOCCmd cmd )
{
  DWORD dwData = 0;
  return IOCTL( cmd,dwData );
};

int TAm9513a::IOCTL( TIOCCmd cmd,DWORD& param )
{
  WORD regH,regL;
  WORD wValue;

  switch( cmd )
  {
    case iocSetTimeTicks:
      // Zerlegung in ein Produkt;
      param = ( DWORD ) ( ( double ) param * fTimeCorrection );
      SplitNumber( param,wLowTicks,wHighTicks );
      ChooseDataPtr( CNT1,CLR );
      WriteData( wLowTicks );
      ChooseDataPtr( CNT2,CLR );
      WriteData( wHighTicks ); // TC Toggle
      param = ( DWORD ) wLowTicks * wHighTicks;
      param = ( DWORD ) ( ( double ) param / fTimeCorrection );
      break;

    case iocSetCounts:
      SplitNumber( param,wLowCounts,wHighCounts );
      param = ( DWORD ) wLowCounts * wHighCounts;
      ChooseDataPtr( CNT3,CLR );
      WriteData( wLowCounts );
      ChooseDataPtr( CNT4,CLR );
      WriteData( wHighCounts ); // TC Toggle
      break;

    case iocStopCCDTiming:
      DisarmC( CTB1 | CTB2 );
      SetToggleOut( CNT2 );
      bIsStopped = TRUE;
      break;

    case iocStartCCDTiming:
      if( !bIsStopped )
        DisarmC( CTB1 | CTB2 );
      ChooseDataPtr( CNT1,CMR );
      // wValue = 0xA121;
      wValue = 0x0121;
      WriteData( wValue );
      ChooseDataPtr( CNT2,CMR );
      // wValue = 0x0202;
      wValue = 0x0222;
      WriteData( wValue );
      bIsStopped = FALSE;
      bIsLatched = FALSE;
      LoadC( CTB1 | CTB2 );
      LoadC( CTB1 | CTB2 );
      SetToggleOut( CNT2 );
      SetToggleOut( CNT4 );
      ClearToggleOut( CNT1 );
      ClearToggleOut( CNT3 );
      ArmC( CTB1 | CTB2 );
      break;

    case iocStartCCDShot:
      if( !bIsStopped )
        DisarmC( CTB1 | CTB2 );
      bIsStopped = FALSE;
      bIsLatched = FALSE;
      LoadC( CTB1 | CTB2 );
      LoadC( CTB1 | CTB2 );
      SetToggleOut( CNT2 );
      SetToggleOut( CNT4 );
      ClearToggleOut( CNT1 );
      ClearToggleOut( CNT3 );
      ArmC( CTB1 | CTB2 );
      break;

    case iocReadTicks:
      if( !bIsLatched )
      {
        DisarmC( CTB1 | CTB2 | CTB3 | CTB4 );
        bIsStopped = TRUE;
        LatchToHoldC( CTB1 | CTB2 | CTB3 | CTB4 );
        bIsLatched = TRUE;
      }
      ChooseDataPtr( CNT1,CHR );
      regL = ReadData( );
      ChooseDataPtr( CNT2,CHR );
      regH = ReadData( );
      param = ( DWORD ) ( wHighTicks - regH )  * ( DWORD ) wLowTicks;
      param += ( DWORD ) ( wLowTicks - regL );
      param = ( DWORD ) ( ( double ) param / fTimeCorrection );
      break;

    case iocReadCounts:
      if( !bIsLatched )
      {
        DisarmC( CTB1 | CTB2 | CTB3 | CTB4 );
        bIsStopped = TRUE;
        LatchToHoldC( CTB1 | CTB2 | CTB3 | CTB4 );
        bIsLatched = TRUE;
      }
      ChooseDataPtr( CNT3,CHR );
      regL = ReadData( );
      ChooseDataPtr( CNT4,CHR );
      regH = ReadData( );
      param = ( DWORD ) ( wHighCounts - regH )  * ( DWORD ) wLowCounts;
      param += ( DWORD ) ( wLowCounts - regL );
      break;

    case iocConfigureAcustic:
      ChooseDataPtr( CNT5,CLR );
      WriteData( ( WORD ) param );
      LoadAndArmC( CTB5 );
      break;

    case iocStartCounting:
      DisarmC( CTB1 | CTB2 | CTB3 | CTB4 );
      bIsStopped = FALSE;
      bIsLatched = FALSE;
      LoadC( CTB1 | CTB2 | CTB3 | CTB4 );
      DelayTime( 1 );
      LoadC( CTB1 | CTB2 | CTB3 | CTB4 );
      ClearToggleOut( CNT1 );
      ClearToggleOut( CNT3 );
      SetToggleOut( CNT2 );
      SetToggleOut( CNT4 );
      DelayTime( 1 );
      ArmC( CTB1 | CTB2 | CTB3 | CTB4 );
      break;

    case iocIsCountingReady:
      switch( 0x14 & ReadStatus( ) )
      {
        case 0x14: param = 0; break; // no counter ready
        case 0x10: param = 1; break; // time counter ready
        case 0x04: param = 2; break; // impulse counter ready
        default: param = 3;
      }
      return ( int ) param;

    case iocStopCounting:
      DisarmC( CTB1 | CTB2 | CTB3 | CTB4 );
      bIsStopped = TRUE;
      break;
  }
  return TRUE;
};

BOOL TAm9513a::SplitNumber( DWORD value,WORD& low,WORD& high )
{
  LONG f1,f2,f,fail;
  int cnt;

  // in case of any changes notify the function IOCTL()
  if( value < 0xF000 )
  {
    high = 1;
    low = ( WORD ) value;
    return TRUE;
  }
  f = value;
  f1 = 4;
  f2 = value / f1;
  cnt = 1000;
  while( cnt-- )
  {
    fail = f - f1 * f2;
    if( ( f1 > 0xE000 )  || ( f2 > 0xE000 ) )
    {
      if( f1 < f2 )
      {
        f1 *= 2;
        f2 /= 2;
      }
      else
      {
        f2 *= 2;
        f1 /= 2;
      }
      continue;
    }
    if( !fail )  break;
    if( f1 < f2 )
      f2 += fail / f1;
    else
      f1 += fail / f2;
    if( ( f1 > fail )  && ( f2 > fail ) )
      break;
  }
  if( f1 < f2 )
  {
    if( f1 < ( fail * 2 ) )
      f2++;
    high = ( WORD ) f1;
    low = ( WORD ) f2;
  }
  else
  {
    if( f2 < ( fail * 2 ) )
      f1++;
    high = ( WORD ) f2;
    low = ( WORD ) f1;
  }
  if( !cnt )
    MessageBox( GetFocus( ),"Fehler beim Zerlegen !","Am9513a",MBINFO );
  return TRUE;
};

DWORD TAm9513a::GetTicksPerSecond( void )
{
  return 100000l;
};

void TAm9513a::DigitalOut( BYTE data )
{
#ifndef __WIN32__
  outp( nBaseAddr+3,data );
#endif
};

void TAm9513a::DigitalIn( BYTE *data )
{
#ifndef __WIN32__
  *data = inp( nBaseAddr + 2 );
#endif
};

BOOL TAm9513a::SelectChip( BYTE )
{
  WORD value;
#ifndef DosVersion
  char buf[ MaxString ];
#endif

  // Zhler zurcksetzen
  WriteCmd( 0xFF );
  Delay( 50 );
  // Check IO
  ChooseDataPtr( CNT1,CLR );
  WriteData( 0xDAAB );
  ChooseDataPtr( CNT2,CLR );
  WriteData( 0x0000 );
  ChooseDataPtr( CNT1,CLR );
  value = ReadData( );
  if( 0xDAAB != value )
  {
    sprintf( buf,"%s : %X","Rckgabewert",value );
    MessageBox( GetFocus( ),buf,"Device-Fehler Am9513a",MBINFO );
    return FALSE;
  }
  return TRUE;
};

void TAm9513a::ArmC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;
  cmd = 0x20 | counter;
  WriteCmd( cmd );
};

void TAm9513a::LoadC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;
  cmd = 0x40 | counter;
  WriteCmd( cmd );
};

void TAm9513a::LoadAndArmC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;
  cmd = 0x60 | counter;
  WriteCmd( cmd );
};

void TAm9513a::DisarmAndSaveC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;
  cmd = 0x80 | counter;
  WriteCmd( cmd );
};

void TAm9513a::LatchToHoldC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;

  cmd = 0xA0 | counter;
  WriteCmd( cmd );
};

void TAm9513a::DisarmC( BYTE counter )
{
  //*** Zhler-Kombination
  unsigned char cmd;
  cmd = 0xC0 | counter;
  WriteCmd( cmd );
};

void TAm9513a::ClearToggleOut( BYTE counter )
{
  //*** Nur einzelne Zhler
  unsigned char cmd;
  cmd = 0xE0 | counter;
  WriteCmd( cmd );
};

void TAm9513a::SetToggleOut( BYTE counter )
{
  //*** Nur einzelne Zhler
  unsigned char cmd;
  cmd = 0xE8 | counter;
  WriteCmd( cmd );
};


void TAm9513a::ChooseDataPtr( BYTE group,BYTE reg )
{
  // Register adressieren
  WriteCmd( group | reg );
};

WORD TAm9513a::ReadStatus( void )
{
  int addr = nBaseAddr + CMD;
  Delay( IoWaitAm9513a );
#ifndef __WIN32__
  return inp( addr );
#endif
  /*
   _asm {
   mov dx,addr
   and ax,0
   in al,dx
   }
   return _AL;
  */
};

void TAm9513a::WriteCmd( BYTE cmd )
{
  if( nbChipId == DIGITAL )  return;
  int addr = nBaseAddr + CMD;
  Delay( IoWaitAm9513a );
#ifndef __WIN32__
  outp( addr,cmd );
#endif
  /*
   _asm {
   mov dx,addr
   and ax,0
   mov al,cmd
   out dx,al
   }
  */
};

void TAm9513a::WriteData( WORD daten )
{
  char test;
  int addr = nBaseAddr + DATA;
  Delay( IoWaitAm9513a );
  test = ( BYTE ) ( daten & 0x00FF );
#ifndef __WIN32__
  outp( addr,test );
#endif
  /*
   _asm {
   mov dx,addr
   mov ax,daten
   out dx,al
   }
  */
  test = ( BYTE ) ( ( ( WORD ) daten >> 8 )  & 0x00FF );
#ifndef __WIN32__
  outp( addr,test );
  Delay( IoWaitAm9513a );
#endif
  /*
   _asm {
   mov dx,addr
   mov ax,daten
   shr ax,8
   out dx,al
   }
  */
};

WORD TAm9513a::ReadData( void )
{
  WORD value;
  int addr = nBaseAddr + DATA;

  Delay( IoWaitAm9513a );
#ifndef __WIN32__
  value = ( WORD ) inp( addr );
#endif
  /*
   _asm {
   mov dx,addr
   and ax,0
   in al,dx
   mov value,ax
   }
  */
  Delay( IoWaitAm9513a );
#ifndef __WIN32__
  value += ( WORD ) ( ( WORD ) inp( addr )  << 8 );
#endif
  /*
   _asm {
   mov dx,addr
   in al,dx
   shl ax,8
   add value,ax
   }
  */
  return value;
};
// __LastLine__

