#ifndef __AM9513A_H
#define __AM9513A_H

// Chip
const BYTE CHIP1 = 0x0;
const BYTE CHIP2 = 0x1;
const BYTE DIGITAL = 0x2;

const BYTE BIT = 0x4;
const BYTE CODE = 0x5;

// IO-Addressen Offset
const int DATA = 0x0;
const int CMD = 0x1;

// Gruppen
const BYTE CNT1 = 0x01;
const BYTE CNT2 = 0x02;
const BYTE CNT3 = 0x03;
const BYTE CNT4 = 0x04;
const BYTE CNT5 = 0x05;
const BYTE CTRG = 0x07;

// Zaehler-Id Bit's
#define CTB1 0x01
#define CTB2 0x02
#define CTB3 0x04
#define CTB4 0x08
#define CTB5 0x10

// Register der Gruppen
const BYTE CMR = 0x00; // Mode-Register
const BYTE CLR = 0x08; // Load-Register
const BYTE CHR = 0x10; // Hold-Register
const BYTE CHIR = 0x18; // Hold+Inc-Register
const BYTE ALM1 = 0x00; // Alarm-Register 1
const BYTE ALM2 = 0x08; // Alarm-Register 2
const BYTE MMR = 0x10; // Master-Mode-Register
const BYTE STS = 0x18; // Status-Register (8 Bit)
// Funktions-Bit fuer CMR
const WORD CB_UP = 3; // aufwaerts zaehlen
const WORD CB_BCD = 4; // BCD zaehlen
const WORD CB_REPEAT = 5; // automatisch neu starten
const WORD CB_RLorH = 6; // laden von L-Reg oder H-Reg
const WORD CB_ENGATE = 7; // special gate
const WORD CB_FALLEG = 12; // mit steigender Flanke zaehlen
// Funktions-Bit fuer MMR
const WORD MB_CMP1E = 2;
const WORD MB_CMP2E = 3;
const WORD MB_NFOUT = 12;
const WORD MB_16BIT = 13;
const WORD MB_DIS_INC = 14;
const WORD MB_BCD_DIV = 15;
// Funktions-Code fuer CMR (nur Setzen mit |)
const WORD CC_CNTM1 = 0x0000;
const WORD CC_NOGATE = 0x0000;
const WORD CC_AHTCNM1 = 0x2000;
const WORD CC_AHNP1 = 0x4000;
const WORD CC_AHNM1 = 0x6000;
const WORD CC_AHN = 0x8000;
const WORD CC_ALN = 0xA000;
const WORD CC_AHEGN = 0xC000;
const WORD CC_ALEGN = 0xE000;
// Output-Control
const WORD CC_I_OLow = 0x0000;
const WORD CC_A_HTCP = 0x0001;
const WORD CC_TCTOG = 0x0002;
const WORD CC_I_OTRI = 0x0004;
const WORD CC_A_LTCP = 0x0005;

// Quellen-Bezeichner fuer CMR und MMR
const WORD C_CNM1 = 0x0; // F1 MMR
const WORD C_SRC1 = 0x1;
const WORD C_SRC2 = 0x2;
const WORD C_SRC3 = 0x3;
const WORD C_SRC4 = 0x4;
const WORD C_SRC5 = 0x5;
const WORD C_GATE1 = 0x6;
const WORD C_GATE2 = 0x7;
const WORD C_GATE3 = 0x8;
const WORD C_GATE4 = 0x9;
const WORD C_GATE5 = 0xA;
const WORD C_F1 = 0xB;
const WORD C_F2 = 0xC;
const WORD C_F3 = 0xD;
const WORD C_F4 = 0xE;
const WORD C_F5 = 0xF;

#define IoWaitAm9513a 43

typedef enum
{
  iocSetCounts = 1, iocSetTimeTicks,
  iocStartCounting, iocStopCounting, iocIsCountingReady,
  iocReadTicks, iocReadCounts, iocConfigureAcustic,
  iocStopCCDTiming, iocStartCCDTiming, iocStartCCDShot
} TIOCCmd;

class TAm9513a
{
    friend class TGenericDevice;
    friend BOOL TriggerLH(  void );
    friend BOOL WINAPI GrabLineData(  void );
    friend class TCCDScan;
  public:
    TAm9513a(  void );
    virtual ~TAm9513a(  void )
    {};
    int Init(  void );
    int IOCTL(  TIOCCmd, DWORD& );
    int IOCTL(  TIOCCmd );
    void DigitalOut(  BYTE );
    void DigitalIn(  BYTE* );
    DWORD GetTicksPerSecond(  void );
    void LoockUp(  LPCSTR );
    void ClearToggleOut(  BYTE );
    void SetToggleOut(  BYTE );
  protected:
    BOOL SelectChip(  BYTE );
    void WriteCmd(  BYTE cmd );
    void ChooseDataPtr(  BYTE group, BYTE reg );
    void WriteData(  WORD );
    WORD ReadData(  void );
    WORD ReadStatus(  void );
    void LoadAndArmC(  BYTE );
    void LoadC(  BYTE );
    void DisarmAndSaveC(  BYTE );
    void DisarmC(  BYTE );
    void LatchToHoldC(  BYTE );
    void ArmC(  BYTE );
    BYTE nbChipId;
  private:
    int nBaseAddr;
    float fTimeCorrection;
    BOOL SplitNumber(  DWORD, WORD&, WORD& );
    BOOL bIsLatched, bIsStopped;
    WORD wLowTicks, wHighTicks;
    WORD wLowCounts, wHighCounts;
};
#endif
// __LastLine__
