#ifndef __DETECUSE_H
#define __DETECUSE_H


/* Benutzt in c_layer.h/cpp, m_steerg.cpp, m_devcom.h */
typedef enum
{
  CounterDevice = 1, PsdDevice, MonitorDevice, EncoderDevice, GenericDevice, CCDDevice
} TDeviceType;

/* Benutzt in braunpsd.cpp, counters.cpp, m_psd.h */
typedef enum
{ 
  FinalRead = 1, FirstRead, AccumulationRead, IntermediateRead
} THowReadOutPsd;

/* Benutzt in braunpsd.cpp, m_devcom.h, m_psd.h */
typedef enum
{ 
  PsdEnergyData = 3000, PsdPositionData
} TPsdDataType;

/* Benutzt in m_devcom.h */
typedef enum
{ 
  NothingHW = 2000, RadiconHW, BraunHW, GenericPsdHW, Am9513HW, StoeHW, 
      MatroxHW, EncoderHW
} THardware;



#define D_DiffractionScan 307

#if defined ( Build_Counters ) 
#define _COUNTERCLASS _export
#elif defined( Use_Counters ) 
#define _COUNTERCLASS _import
#else
#define _COUNTERCLASS
#endif


typedef struct
{
  WPARAM wpJumpTo;
  float fExposureTime;
  DWORD dwExposureCounts;
  float fFailure;
  int nFunctionId;
  BOOL bStaticFailure;
  int nAddedChannels;
}
TDSettings;


class TCurve;

class __rtti _COUNTERCLASS TDevice
{
//JP  *****************************************************************
//JP  Dieser Kommentar wird in einer spaeteren Version entfernt.
//JP  Ich habe bei allen Funktionen, die im Klassendefinitionskoerper
//JP  vollstaendig definiert werden, das Schluesselwort inline
//JP  entfernt, da diese Funktionen ohnehin implizit inline sind.
//JP  *****************************************************************
    
	friend class TDList;
    friend class TCommonDevParam;
    friend class TCounterWindow;
    friend class TAreaScan;
  public:
    TDevice( void );
    virtual ~TDevice( void );
    virtual int Initialize( void );
    virtual int SetParameters( void ) { return 1; };
    virtual void SetAddedChannels( int ) {};
    virtual int GetAddedChannels( void ) { return 1; };
    virtual float GetMaximumIntensity( void ) { return 1.0; };
    virtual BOOL SetSound( void ) { return TRUE; };
    virtual float GetAngleRange( void ) { return 1.0; }
    virtual float GetAngleRange( TUnitType ) { return 1.0; }
    THardware GetHardware( void ) { return eHardware; };
    virtual int MeasureStart( void );
    virtual int MeasureStop( void );
    virtual int MeasureStopExternal( void ) { return 1; };
    int SetExposureValues( float, DWORD, float );
    void GetExposureValues( float&, DWORD&, float& );
    virtual void SetSpezificParametersDlg( void ) {};
    virtual int PollDevice( void );
    virtual int SingleStepMeasurement( int ) { return 0; };
    virtual int GetData( float& );
    virtual int GetData( TCurve& ) { return 0; };
    virtual int GetData( WORD*, WORD, WORD ) { return 0; };
    virtual UINT GetMaximumChannel( void ) { return 0; };
    virtual void SetAngleStep( float ) {};
    virtual float GetAngleStep( void ) { return 0.0; };
    virtual void SetSignalGrowUp( BOOL ) {};
    virtual BOOL IsSoftOverflow( void ) { return FALSE; };
    virtual BOOL IsHardOverflow( void ) { return FALSE; };
    virtual LPCSTR GetUnitStr( void ) { return ""; }
    virtual float GetWidth( void ) { return 1.0; };
    virtual float GetWidth( TUnitType ) { return 1.0; };
    virtual void SetDataType( TPsdDataType ) {};
    int GetId( void ) { return nDevId; };
    int IsOk( void ) { return bDeviceOk; };
    void UpdateDisplay( void );
    BOOL IsActive( void ) { return bDeviceActive; };
    void SetHost( HWND hwnd ) { hControlWnd = hwnd; };
    float GetSigma( void ) { return fSigma; };
    virtual BOOL PushSettings( void );
    virtual BOOL PopSettings( void );
    BOOL SelectMFunction( int mf ) { nFunctionId = mf; return TRUE; };
    int GetMFunction( void ) { return nFunctionId; };

    //! methode realisiert intensitaetsmessung waehrend dem zeitintervall
    //! zwischen zwei aufrufen der Methode
    static CALLBACK EventHandler( UINT, UINT, DWORD, DWORD, DWORD );


    //! initialisiert ein periodisches aufrufen der methode EventHandler

	  virtual BOOL InitializeEvent( HWND, int );


    //! liefert die durch EventHandler gemessene Intensitaet im Parameter zurueck

    virtual BOOL GetEventIntensity( float& );


    //! macht dem Detektor das Fenster bekannt, an das er innerhalb von
    //! EventHandler seine Nachrichten schicken soll
	  void SetEventHost(HWND);

    //! loescht Detektor-Event
	  void KillEvent( void );


//JP  *****************************************************************
//JP  Diese Attribute muessten eigentlich mindestens protected sein.
//JP  Auf sie wird aber aus anderen Subsystemen direkt zugegriffen.
//JP  Bis zur Schaffung eines neuen Interfaces muessen sie also public
//JP  bleiben.
	BOOL bDeviceOpen; // Geraet ist in irgendeiner Form aktiv
    char Characteristic[ MaxString ];
    double dRealTime;
    BOOL bDataValid;
    float fIntensity;
//JP  *****************************************************************

//JP  *****************************************************************
//JP  Dieser Abschnitt wird in einer spaeteren Version entfernt.
//JP  Diese Attribute werden im Projekt nicht verwandt
//JP  int nPeakWindow; // in Prozent
//JP  int nPeakLevel; // in Prozent
//JP  BOOL bAnalysator;
//JP  BOOL bDiskriminator;
//JP  *****************************************************************

//JP  *****************************************************************
//JP  Dieser Abschnitt wird in einer spaeteren Version entfernt.
//JP  Attribute sind jetzt protected.
//JP  BOOL bSound;
//JP  BOOL bStaticFailure; // Mess-Fehler auf bestimmtem Wert halten
//JP  int nFunctionId;
//JP  float fSigma;
//JP  *****************************************************************

  protected:

//JP  *****************************************************************
//JP  Dieser Kommentar wird in einer spaeteren Version entfernt.
//JP  vorher: double CalcTime( DWORD );
	static double CalcTime( DWORD mssum) // const 
	{ return ( (double)mssum/1000L + ((double)(mssum%1000)) / 1000.0 ); };
//JP  *****************************************************************

	void SetDisplay( HWND hwnd ) { hDisplayWnd = hwnd; };
    HWND GetDisplay( void ) { return hDisplayWnd; };
    HWND hDisplayWnd;
    HWND hControlWnd;
    BOOL bDeviceActive;
    BOOL bDeviceOk;
    WPARAM wpJumpTo;
    DWORD dwElapsedTime, dwStartTime;
	BOOL bSound;
	BOOL bStaticFailure; // Mess-Fehler auf bestimmtem Wert halten
    UINT uAskTime;
    BOOL bDebug;
    HWND hWndFrame;
    TDSettings Settings;
    float fExposureTime;
    DWORD dwExposureCounts;
    float fFailure;
    THardware eHardware;
    int nFunctionId;
    float fSigma;

//JP  *****************************************************************
//JP  Dieser Kommentar wird in einer spaeteren Version entfernt.
//JP  Diese vier Hardware-Limits sind konstante Eigenschaften der
//JP  jeweiligen Hardware. Ihre Funktionalitaet sollte also eher
//JP  durch virtuelle Funktionen realisiert werden.
    DWORD dwLowerCountBound;
    DWORD dwUpperCountBound;
    float fLowerTimeBound;
    float fUpperTimeBound;
//JP  *****************************************************************

  private:
    static int nMaxId;
    int nDevId;
};


class __rtti _COUNTERCLASS TDList
{
//JP  *****************************************************************
//JP  Dieser Kommentar wird in einer spaeteren Version entfernt.
//JP  Ich habe bei allen Funktionen, die im Klassendefinitionskoerper
//JP  vollstaendig definiert werden, das Schluesselwort inline
//JP  entfernt, da diese Funktionen ohnehin implizit sind.
//JP  *****************************************************************

  public:
    TDList( int, HINSTANCE );
    ~TDList( void );
    TDevice* DP( void );
    TDevice* DP( int );
    TDeviceType ParsingAxis( LPSTR );
    BOOL SetDevice( int );
    BOOL IsValidId( int id ) { return ( ( id >= 0 )  && ( id <= nLastDevice ) ); };
    BOOL DeleteDevices( void );
//JP  *****************************************************************
//JP  Dieser Abschnitt wird in einer spaeteren Version entfernt.
//JP  CalibrateDlg wird im gesamten Projekt nicht verwendet.
//JP  void CalibrateDlg( TCurve& );
//JP  *****************************************************************
    int GetDevice( void ) { return nActiveDevice; };
    int LastId( void ) { return nLastDevice; };

    HINSTANCE GetModuleHandle( void ) { return hDInstance; };
    BOOL InitializeModule( void );
    BOOL SaveModuleSettings( void ) { return TRUE; };
    void SetParametersDlg( void );
    int GetIdByName( TDeviceType );
    BOOL IsDeviceValid( TDeviceType );
  private:
    int MD_Monitor;
    int MD_Encoder;
    int MD_Psd;
    int MD_Counter;
    int MD_Generic;
    HINSTANCE hDInstance;
    int nMaxNumber;
    int nLastDevice; // points the last device
    int nActiveDevice; // points the actual device
    TDevice** aDevice;
};

typedef TDList FAR * LPDList;
typedef TDevice FAR * LPDevice;


class __rtti TPsd : public TDevice
{
    friend class TPsdParameters;
  public:
    TPsd(  void );
    virtual ~TPsd(  void );
    virtual int Initialize(  void );
    BOOL PushSettings(  void );
    BOOL PopSettings(  void );
    virtual int SetEnergyRange(  UINT, UINT )
    { return R_OK; };
    virtual int GetEnergyRange(  UINT&ler, UINT& her )
    { her = 1; ler = 0; return R_OK; };
    void SetAddedChannels(  int ach );
    void SetSpezificParametersDlg(  void );
    void SetAngleRange(  float );
    int SetParameters(  void );
    void SetSignalGrowUp(  BOOL gup )
    { bSignalGrowUp = gup; };
    void SetAngleStep(  float stp )
    { fAngleStep = stp; };
    virtual int GetBufferSize(  void )
    { return nChannelNumber+1; };
    float GetAngleStep(  void )
    { return fAngleStep; };
    float GetAngleRange(  void );
    float GetAngleRange(  TUnitType );
    BOOL IsSoftOverflow(  void )
    { return bSoftOverflow; };
    BOOL IsHardOverflow(  void )
    { return bHardOverflow; };
    float GetWidth(  TUnitType );
    float GetWidth(  void )
    { return fAngleWidth; };
    int GetAddedChannels(  void )
    { return nAddedChannels; };
    int GetChannelNumber(  void )
    { return nChannelNumber; };
    int GetData(  TCurve& );
    int GetData(  float& );
    int GetData(  WORD*, WORD, WORD )
    { return 0; };
    UINT GetMaximumChannel(  void );
    float GetMaximumIntensity(  void )
    { return (  dwMaxCounts / dRealTime ); };
    virtual int MeasureStart(  void );
    virtual int MeasureStop(  void );
    int MeasureStopExternal(  void );
    virtual int PollDevice(  void );
    LPCSTR GetUnitStr(  void )
    { return ( LPCSTR ) szUnit; }
  protected:
    char szUnit[ 20 ];
    int nBaseAddr;
    TUnitType eUnit;
    HGLOBAL hCountBuf;
    BOOL bHVRegelung; // boolscher Wert
    int nCountBufItems;
    int nMinChannel;
    int nMaxChannel;
    int nChannelNumber;
    DWORD dwMaxCounts, dwIntegratedCounts;
    UINT uMaximumChannel;
    BOOL bReadyForRead;
    BOOL bHardOverflow;
    BOOL bReadLeftFirst;
    long lSeconds;
    long lWaitTime;
    BOOL bFirstReadOut;
    BOOL bSignalGrowUp;
  private:
    float fAngleStep;
    float fAngleWidth;
    int nAddedChannels;
    int s_nAddedChannels;
    float s_AngleStep;
    virtual int PsdInit(  void )
    { return R_OK; };
    virtual int PsdStart(  void )
    { return R_OK; };
    virtual int PsdStop(  void )
    { return R_OK; };
    virtual int PsdReadOut(  THowReadOutPsd );
    virtual int PsdRead(  int, int, int, LPWORD )
    { return R_OK; };
    BOOL bSoftOverflow;
    float OverflowIntensity;
};


#ifdef __cplusplus
extern "C"
{
#endif
  LPDList WINAPI GetCounterListPtr(  void );
  int WINAPI dlGetDevice(  void );
  BOOL WINAPI dlSetDevice(  int );
  TDeviceType WINAPI dlParsingDevice(  LPSTR );
  int WINAPI dlGetIdByName(  TDeviceType );
  BOOL WINAPI dlIsDeviceValid(  TDeviceType );
  LPCSTR WINAPI dlGetVersion(  void );
  HINSTANCE WINAPI dlGetInstance(  void );
  int WINAPI dMeasureStart(  void );
  int WINAPI dMeasureStop(  void );
  BOOL WINAPI dSetExposureValues(  float, DWORD, float );
  BOOL WINAPI dGetExposureValues(  float&, DWORD&, float& );
  BOOL WINAPI InitializeCountersDLL(  void );

#ifdef __cplusplus
}
#endif

#endif
// __DETECUSE_H
// __LastLine__