/*
Datei     : radicohw.h
Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2001
            Institut fuer Informatik,
            Humboldt-Universitaet Berlin
Inhalt    : Interface der Klasse TRadiconHW
*/

/* TRadiconHW macht die Dateien 
  
  kmpt1.c
  prkmpt1.h
  kisl1.c
  radicon.h
  dfkisl.h

obsolet. 
Die gesamte Funktionalitt dieser Dateien steht jetzt in einer OO-Version
zur Verfuegung. 

Die Implementation orientiert sich dabei an der Funktionalitaet der 
vorgenannten C-Implementation, versucht dabei aber, mehr C++ als C 
darzustellen. (kein inline-Assembler, stream-I/O, Zugriffsschutz, 
keine #define's, const's, class enum's, ... ).
Ich hoffe, dass ich damit insbesondere die Portabilitaet verbessert habe.
*/

#ifndef __RADICOHW_H
#define __RADICOHW_H

#include <windows.h>
//#include "hwio.h"

// TRadiconHW kapselt die Kommunikation mit der Radicon-Controller-Karte.
class TRadiconHW {
  typedef unsigned long ulong;

private:
  TIOPort control;							// Kontroll-Port
  TIOPort data;									// Daten-Port
  BOOL fakeDevice;							// Kommunikation nur simulieren

protected:
  static DWORD CurrentTime();		// Systemzeit in ms
  
	// Bereitschaft des Controllers zum Senden von Daten ermitteln
	int ReadStatus();							

  // Bereitschaft des Controllers zum Empfangen von Daten ermitteln
	int WriteStatus();
  

	int ReadData(int&);						// Daten-Byte vom Controller lesen
  int WriteData(int);						// Daten-Byte zum Controller senden
  

  // Nachricht vom Controller empfangen
	int ReceiveMessage(int, unsigned char*, int&, unsigned int maxlen=25 );
	
	// Nachricht zum Controller senden
  int TransmitMessage(int, char const* buf=NULL, unsigned int len=0);

public:
  TRadiconHW( int ctrlPort, int dataPort, BOOL fake=FALSE );

  enum readoutMode
  {
    final  =0x9,  // Ergebnis-Messzeit und -impulse auslesen (vorh. CF9)
    interim=0xb,  // Aktuelle Messzeit und -impulse auslesen (vorh. CFB)
  };

  enum operationMode
  {
    fixedExposureTime =0x1,  // Impulse waehrend ExposureTime zaehlen
    fixedImpulseCounts=0x2,  // Zeit messen, wenn Impulszahl vorgegeben
    intensimeterMode  =0x3,  // Intensimeter
    stopDevice        =0x5,  // Geraet stoppen
  };

  // Controller-Einstellungen setzen
	int SetParameters( 
		  unsigned short upperThresh, unsigned short lowerThresh,
      int highVoltage,double exposureTime, unsigned long impulseCount,
      BOOL sound, int energyIntegral=1, int Interrupt=0 );
															
  
	// Controller-Betriebsmodus einstellen
	int SetOperationMode( operationMode fun );

  // Messwerte auslesen
	int GetValues( readoutMode mode, double &exposureTime, unsigned long &impulseCount );
  
	// Controller-Firmware in den Controller laden
	int FirmwareUpload ( char const *bufmsg, unsigned int len);
  
	// Controller zuruecksetzen
	void Reset();
  
	// Ermitteln, ob die Kommunikation mit dem Controller nur simuliert wird
	BOOL FakeDevice();
  
	// Einstellen, ob die Kommunikation mit dem Controller simuliert werden soll
	void FakeDevice(BOOL);
};

#endif // #ifndef __RADICOHW_H
