#ifndef __DIFRKMTY_H
#define __DIFRKMTY_H


class TDevice;
typedef TDevice FAR * LPDevice;

typedef enum
{
  MonitorIntensity = 1, AbsorberPosition, PeakPosition, IntegralIntensity,
      MeasurementTime
} TNotifyValue;


//! Speichern, ohne nach Dateinamen zu fragen
const int NoAskForSave = 0x9900;

//! Speichern mit Abfrage des Dateinamens
const int SaveWithNewName = 0x9901;

//! Anzahl der max. unterschiedlichen Schrittweiten (dyn. Schrittweite)
const int nMaxScaleIdx = 10;


class TScanParameters
{
    //! einlesen der Ini-daten zu [Scan] und setzen von default-werten fuer
    //! StepScan
public:
    TScanParameters( void );

    //! beinhaltet Nummer der aktuellen Achse
    int nMotor;

    //! Speicherformat fuer Kurven
    TSaveFormat eSaveFormat;

    //! Art des aktuellen Scan(StandardScan, Omegea2Theta o. ContinuousScan)
    TxScanType eScanType;

    //! Speichern nach Beendigung der Messung
    BOOL bSaveOnReady;

    //! Anzahl der Messpunkte(nur bei Cont.Scan)
    int nCollectionAmount;

    //! Startwinkel des Scans
    double dArgumentMin,
    //! Endwinkel des Scans
    dArgumentMax,
    //! Schrittweite waehrend des Scans
    dArgumentWidth;

    //! Geschwindigkeit des aktuellen Motors
    double dSpeed;

    //! aktueller Detektor zum Messen
    LPDevice Device;

    int nMFunctionId; // Typ der Mess-Funktion
    WORD nNumberCycle; // Anzahl der Mess-Zyklen
	 float fFailure; // Zur Konfiguration des Zaehlers
    float fMaxTime; // Maximale Messzeit
    DWORD dwMaxCounts; // Maximale Anzahl Impulse

    //! Detektor fuer Monitor
    LPDevice Monitor;

    //! Speichern waehrend der Messung
    BOOL bSaveContinuous;

    //! Monitor-detektor benutzen
    BOOL bMonitorUsed;

    //! gibt an ob dynamische Schrittweitensteuerung aktiv
    BOOL bDynamicStepWidth;

    //! Anzahl der verschied. Schrittweiten
    int nStepScaleLevels;

    //! Liste enthaelt in den ersten 10 werten die intensitaeten und in den
    //! zweiten 10 werten den zughoer. Schrittweitenfaktor
    float Scaler[ 20 ];

    //! gibt 2ThetaStartposition fuer einen zu fahrenden Stepscan an beim
    //! O2T-Scan an
    double d2ThetaStart;


};

class TScan : public TScanParameters, public TPlotData
{
    friend class TSetScanParameters;
    friend class TSteering;
	 friend class TScanCmd;
    friend class TSetupScanCmd;
    friend class TSaveDataCmd;
	 //! Aenderung Dynamische Schrittweite
	 friend class TSetupDynamicStep;

public:
    TScan( void );
    ~TScan();

protected:
    //! laden einer gespeicherten Kurve (.crv/.bk/.dtn)
    //! ?? input.par. nicht impl.
    //! Rueckkehrcode false wenn Fehler beim oeffnen der Datei, fehlerhaften
    //! Headerinfo. oder keine Eintraege nach[DATA] sonst true
    BOOL LoadOldData( int );

    //! speichern einer .crv Datei
    //! ask== NoAskForSave(0x9900) => speichert unter Namen aus FileName
    //! ohne Abfrage
    //! == SaveWithNewName(0x9901) => speichert unter Namen, der in der
    //! Dialogbox erst noch enzugeben ist
    //! Rueckkehrcode False wenn Fehler bei anlegen der Datei oder bei
    //! speichern der Headerinfo. sonst true
    BOOL SaveFile( int );

    //! haengt aktuelles tripel der Messkurve an Datei mit FileName an
    //! aufruf nur bei kontinuierlichem Speichern
    //! Rueckkehrcode False wenn fehler bei oeffnen der Datei sonst true
    BOOL UpdateFile();

    //! ?? scheinbar ueberfluessig
    //! rueckkehrcode: immer true
	 BOOL SaveDataBase( void );

    //! darstellen des StepScanfenster
    void Create( HWND hwnd );

    //! erhaelt Messpunkt, stellt ihn dar, speichert ihn ggf.
    void CounterSetRequest( LPARAM );

    //! behandelt Ende des Messvorgangs
    void SteeringReady( LPARAM );

    //! passt x+y-Scale des Koord.sys an neue Werte an
    void SetRanges( void );

    //! ?? scheinbar ueberfluessig
    void DrawMenu( void );

    //! loescht aktuelle Kurve + baut StepScanfenster neu auf
    //! rueckkehrcode: immer true
    BOOL New( void );

    //! laden der Messungsdaten aus einer .crv bzw. .bk datei
    //! wird von TScan::LoadOldData() in m_scan.cpp aufgerufen
    //! hFile ist Filehandle der Messdatei
    //! Rueckkehrcode False wenn Datei zu kurz oder [HEADER] nicht vorhanden
	 //! sonst true
    BOOL LoadMeasurementInfo( HFILE );

	 //! Speichern bzw. Aktualisieren der Headerinformation
    //! Par.==0 => neu erstellen
    //! ==1 => aktualisieren
    //! Rueckkehrcode False wenn Fehler bei speichern sonst true
    BOOL SaveMeasurementInfo( BOOL );

    //! aufruf des Stepscanvorgangs
    void InitializeTask( WPARAM, LPARAM );

    //! behandelt einige Menuekommandos
    void InitializeDlg( WPARAM, LPARAM );

    //! unterbricht Scanvorgang
    void Interrupt();

    //! ?? scheinbar ueberfluessig
    void StopMeasurement();

private:
    //! ?? scheinbar ueberfluessig
    BOOL bMacroRunning;

    //! setzt Titel im Stepscanfenster
    //! rueckkehrcode: immer true
    BOOL SetTitle( void );

    //! ?? scheinbar ueberfluessig
    BOOL NewFile();

    //! ?? scheinbar ueberfluessig
    void ExecuteNextCmd( void );

    //! Test ob Stepscanfenster schliessbar
    //! Rueckkehrcode False wenn kein Fenster geoeffnet war, sonst immer True
    BOOL CanClose( void );

    //! setzt Untermenue an Mausposition
    void rButtonDown( LPARAM );

    //! liefert Klassennamen
    LPCSTR ClassName()
    {
        return "Scan";
    };

    //! liefert Charakteristik, ?? scheinbar ueberfluessig
    LPCSTR GetCharacteristic()
    {
        return "Curve";
    };

    //! ?? scheinbar ueberfluessig
    double GetArgumentMin( void )
	 {
        return dArgumentMin;
    };

    //! ?? scheinbar ueberfluessig
    double GetArgumentMax( void )
    {
        return dArgumentMax;
    };

    //! soll Scanstart vor Aufruf einer Einstellungsbox verhindern
    //! (Fehler 2)
    BOOL bScanSetupOk;

};

//! struktur haelt Info zu reportkurven
//! (welche zusatzinfo existieren und welche nicht)
typedef struct
{
    BOOL bMonitor;//! Monitorintensitaet
    BOOL bAbsorber;//! Absorberposition
    BOOL bTime;//! reale Messzeit
    BOOL bIntegral;//! integrale Intensitaet
}
TReportUse;

class TAreaScanParameters
{

public:
	 TAreaScanParameters( void );

	 //! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
	 int nOmega, nTheta;

	 //! enthaelt Anfangs- und Endwinkel sowie die Schrittweite des Omegamotors
	 //! fuer einen Areascan
	 double dOmegaMin, dOmegaMax, dOmegaWidth;

	 //! enthaelt Anfangs- und Endwinkel , die Schrittweite sowie den Offset des
	 //! Thetamotors fuer einen Areascan
	 //! NEU dThetaStep
	 double dThetaMin, dThetaMax, dThetaWidth, dThetaOffset;

	 //! gibt das Schrittverhaeltnis zwischen Omega und Theta an
	 double dMoveRelation,
	 //! gibt den Thetabereich eines Spektrums an
	 dThetaWindow;

	 //! gibt den Scantyp an (omega2Theta/StandardScan)
	 TxScanType eScanType;

	 //! gibt an in welcher Form die Messwerte in Datei abgelegt werden sollen
	 //! (nur StandardFile verwendet, erweitert um bkFile, dtnFile)
	 TSaveFormat eSaveFormat;

	 //! Speichern nach Beendigung der Messung
	 BOOL bSaveOnReady;

	 //! Speichern waehrend der Messung
	 BOOL bSaveContinuous;

	 //! Aenderung DB_Zerlegung
	 BOOL bSaveEachScan;

	 //! gibt an ob ein Psd angeschlossen
	 BOOL bPsdDevice;

	 //! bisher ueberfluessig (bei anderen Speicherformaten verwendet)
	 BOOL bThetaAddsPsd;

	 //! gibt ausgewaehlten Detektor an
	 LPDevice Device;

	 //! zeigt an ob AreaScan mit Absorber durchzufuehren
	 BOOL bAbsorberUsed;

	 //! gibt an wieviele Kanaele des Psd zu einem zusammenzufassen sind
	 int nAddedChannels;

	 float fFailure; // Zur Konfiguration des Zaehlers
	 float fMaxTime; // Maximale Messzeit
	 DWORD dwMaxCounts; // Maximale Anzahl Impulse

	 //! gibt ausgewaehlten Monitordetektor an (ist bisher nicht auswaehlbar)
	 LPDevice Monitor;

	 //! zeigt an ob AreaScan mit Monitor durchzufuehren (ueberfluessig siehe
	 //! Monitor)
	 BOOL bMonitorUsed;

	 //! gibt an welche zusatzdaten zu einem Spektrum im Scanreport zu speichern
	 //! sind , zur zeit nur IntegralIntensity moeglich
	 TNotifyValue eNotifyValue;

	 //! setzt den Detektor, dessen id-nummer dem Param. id entspricht
	 //! dabei wird bPsdDevice gesetzt, wenn der Detektor ein Psd ist
	 void SetDevice( int );

	 //! gibt die gewuenschte Thetastartposition bei O2T-Scans bzw. die feste
   //! position beim Standardscan an
	 double d2ThetaStart;

	 //! Aenderung DB_Zerlegung
	 char FirstFile[ MaxString], LastFile[ MaxString ];
	 int MaxFiles;

	 //! gibt den Omega- bzw. Thetaoffset an (differenz zwischen gewuenschtem und
   //! realem winkel)
	 double dOffsetOmega,dOffsetTheta;

   //! gibt Winkelwert an, der sich aus der Nummer des Messkanals und der
   //! breite eines Psd-kanals ergibt; um diesen winkelwert muss man verschieben,
   //! um den Messkanal auf der 2Thetaposition zu positionieren
   double dPsdOffset;

   //! gibt Messkanal an ; Kanal des Psd der sich immer auf der 2Thetaposition
   //! befinden soll
   int nMeasurementChannel;

   //! gibt an welche Zusatzinfos zu der Datenbasis anzulegen sind bzw. bei
   //! nachgeladenen welche Infos existieren
   TReportUse ReportUse;

	 //! neu akk
	 //! gibt art der Darstellung waehrend eines AreaScans mit Psd an
	 //! akkumulierte Anzeige (TRUE) oder
 	 //! Anzeige am Ende der Messzeit (FALSE)
   BOOL bAccumulatedDisplay;
};

class TAreaScan : public TAreaScanParameters, public TPlotData 
{
	 friend class TSteering;
	 friend class TSetupScanCmd;
	 friend class TScanCmd;
	 friend class TSetupAreaScanCmd;
	 friend class TAreaScanCmd;
	 friend class TSaveDataCmd;
	 friend class TChooseScan;
	 friend class TAquisition;

public:
	 TAreaScan( void );
	 ~TAreaScan( void );

protected:

	 //! Funktion wird bei WM_CREATE aus WndProc aufgerufen
	 //! testet ob weniger als 5 Fenster geoeffnet sind liefert dann ein true
	 //! als Rueckkehrcode ansonsten ein false
	 //! wahrscheinlich ueberfluessig (wird geerbt)
	 BOOL CanOpen( void );

	 //! test ob Fenster geschlossen werden kann
	 //! -test ob Messung aktiv, wenn ja, dann zuerst Messung unterbrechen
	 //! (Interrupt())
	 //! -test ob geaenderte Daten, wenn ja, dann Speicher-Anfrage
	 //! Rueckkehrcode False wenn kein Fenster goeffnet war sonst immer True
	 //! wahrscheinlich ueberfluessig (wird geerbt)
	 BOOL CanClose( void );

	 //! liest gespeicherte Kurve (.psd)
	 //! ?? inputparam nicht implem.
	 //! Rueckkehrcode false wenn Fehler beim oeffnen der Datei, Extension ungl.
	 //! .psd, fehlerhaften Headerinfo. oder keine Eintraege nach[DATA]
	 //! sonst auch false (fehler)
	 BOOL LoadOldData( int );

	 //! Darstellen des AreaScan-Fenster
	 void Create( HWND );

	 //! wahrscheinlich ueberfluessig da vererbt von TMDIWindow
	 void DrawMenu( void );

	 //! Funktion behandelt Fenstermenuepunkte
	 LRESULT CallLocalAction( WPARAM, LPARAM );

	 //! aufruf wenn rechte Maustaste gedrueckt
	 //! Fenstermenue an aktueller Mausposition ausgeben
	 //! eingabeparam. aktuelle Mauspos.low(8) x-pos
	 //! high(8) y-pos.
	 void rButtonDown( LPARAM );

	 //! aufruf wenn linke Maustaste losgelassen wurde(d.h. Endpunkt erreicht)
	 //! wird in 2 Faellen aktiv
	 //! - messen von entfernung zwischen zwei Punkten ueber TPlotData
	 //! - bei Dataaquisition(nMouseAction==100) zusaetzlich Kommando senden
	 void lButtonUp( WPARAM, LPARAM );

	 //! aufruf wenn linke Maustaste gedrueckt wurde
	 //! (d.h. Startpunkt wurde gewaehlt)
	 //! wird in 2 Faellen aktiv
	 //! - messen von entfernung zwischen zwei Punkten (nMouseAction=0)
	 //! - bei Dataaquisition (nMouseAction=100)
	 void lButtonDown( WPARAM, LPARAM );

	 //! wird aufgerufen aus m_main.cpp bei cm_InitializeScan
	 //! startet den Scanvorgang
	 void InitializeTask( WPARAM, LPARAM );

	 //! behandelt die Kommandos 'Setup Measurement...' und 'Inquire Data'
	 //! des Kindfenster-menue ; aufruf aus m_main.cpp DoCommandsChild()
	 void InitializeDlg( WPARAM, LPARAM );

	 //! wahrscheinlich ueberfluessig
	 void StopMeasurement( void );

	 //! Fkt. wird bei unterbrechen/fortsetzen eines AreaScans aufgerufen oder
	 //! zum ausschalten einer kont. Anzeige
	 void Interrupt( void );

	 //! ??
	 //! Aufruf aus CallLocalAction bei ExternalSync
	 //! Dialogbox unvollstaendig + gesamte Funktionalitaet ExternalSync nur in
	 //! Ansaetzen implementiert
	 void ExternSychronized( void );

	 //! aufruf aus CallLocalAction zur PsdCalibrierung
	 void CalibratePsd( void );

	 //! bildn. Darstellung ausschl. in TPlotData verwendet
	 BOOL SetMeasurementArea( TKSystem & );

	 //! loescht aktuelle datenbasis + baut AreaScanfenster neu auf
	 //! Rueckkehrcode immer True
	 BOOL New( void );

	 //! Funktion speichert einen Messdatensatz
	 //! 1.Parameter gibt an ob neuer Dateiname zu waehlen ist (SaveWithNewName)
	 //! oder ob automatisch unter altem Namen (NoAskForSave) gespeichert werden
	 //! soll
	 //! Rueckkehrcode False wenn Fehler bei anlegen der Datei oder bei
	 //! speichern der headerinfos sonst immer true
	 BOOL SaveFile( int );

	 //! speichert die zur Datenbasis gehoerigen zusatzinfos in einer Datei
	 //! unter dem aktuellen Dateinamen mit der Extension .rep ab
	 //! Rueckkehrcode immer true
	 BOOL SaveReport( void );

	 //! Funktion laedt die zu einer .psd datei gehoerigen zusatzinfos
	 //! Rueckkehrcode False wenn Fehler bei oeffnen der Datei, wenn weniger
	 //! als 3 Punkte in der Datei waren oder wenn header defekt
	 //! true sonst
	 BOOL LoadReport( void );

	 //! zeichnet in abhaengigkeit des eingestellten Darstellungstyps das
	 //! AreaScanfenster neu
	 void UpdateWnd( int );

	 //! haengt aktuelles Kurve an Datei mit FileName an
	 //! aufruf nur bei kontinuierlichem Speichern
	 //! Rueckkehrcode immer true
	 BOOL UpdateFile( void );

	 //! setzt die Achsen-min. und -max. abhaengig von der Darstellungsart
	 void SetRanges( void );

	 //! Funktion liefert den absoluten Thetawert als Rueckkehrcode
	 //! 1.Param. scid gibt Kurvennr. an, deren absoluter Thetawert gesucht ist
	 //! nur bei RL-Bitmap benutzt
	 double GetThetaOffset( int );

	 //! liefert den Klassennamen zurueck
	 LPCSTR ClassName()
	 {
		  return "AreaScan";
	 };

	 //! liefert Klassencharakteristik (nicht benutzt)
	 LPCSTR GetCharacteristic()
	 {
		  return "AreaScan";
	 };

	 //! behandelt eigentlichen Scanablauf sowie die kont. Anzeige ,Calibrate...
	 //! aufruf aus m_main.cpp wenn cm_CounterSet
	 void CounterSetRequest( LPARAM );

	 //! diese Fkt. schaltet die kont. Spektrumsanzeige fuer Psd an oder aus
	 //! 1.Param. type== PsdEnergyData(3000) => kont. EnergieSpektrum an/aus
	 //! PsdPositionData(3001) => kont. ImpulsSpekrum an/aus
	 //! (99) => schaltet kont. Spektrum aus
	 //! (0) => schaltet kont.Spektrum aus und dieses spaeter
	 //! wieder ein
	 //! nicht implementiert aber benutzt
	 void ShowSensorContinuous( BOOL );

	 //! wird augerufen nachdem AreaScan abgeschlossen wurde
	 //! aus m_main.cpp bei cm_SteeringReady
	 void SteeringReady( LPARAM );

	 //! scheinbar ueberfluessig
	 double GetArgumentMin( void )
	 {
		  return dOmegaMin;
	 };
	 double GetArgumentMax( void )
	 {
		  return dOmegaMax;
	 };

	 //! nur bei benutzung von 0 dim. Detekt. von Bedeutung
	 //! zeigt an ob der Detektor den Thetabereich durchlaufen hat
	 BOOL bThetaScanCompleted;

private:

    //! gibt Nummer der Kurve in Datenbasis an, die gerade aktiv ist
    int nActiveScan;

    //! scheinbar ueberfluessig
    HWND hOldControlWnd;

    //! zeigt an, ob kont. Spektrumsanzeige aktiv ist
    BOOL bShowPsdContinuous;

	 //! zeigt an, ob die Psdkalibrierung gerade durchgefuehrt wird
    BOOL bCalibrationActive;

    //! scheinbar ueberfluessig
    BOOL bMacroRunning;

    //! scheinbar ueberfluessig
    BOOL bNewCurveNeeded;

    //! setzt Titel des AreaScanFensters
    //! Rueckkehrcode immer True
    BOOL SetTitle( void );

	 //! Funktion speichert Headerinformationen zu einem Areascan in einer Datei
    //! 1.Parameter gibt an, ob die Datei neu erstellt(=0) werden soll oder nur
    //! aktualisiert (ungl. 0) werden soll
    //! Rueckkehrcode false wenn probleme beim erstellen bzw. oeffnen der Datei
    //! sonst immer true
    BOOL SaveMeasurementInfo( BOOL );

    //! laden der Messungsinfos aus einer .psd datei
    //! hFile ist Filehandle der Messdatei
    //! Rueckkehrcode False wenn Datei zu kurz ,[HEADER] nicht vorhanden,
    //! Filetyp ungl. 1 oder OmegaStep = 0
    //! true sonst
    BOOL LoadMeasurementInfo( HFILE );

    //! zeigt an, ob gerade das erste Spektrum eines AreaScans aktiv ist
	 //! wird benutzt, da die Datenbasis bereits mit einer Kurve init. wird
    //! aber in den folgendenSchritten immer eine neue Kurve anlegen muss
    BOOL bFirstScan;

    //! Kurve enthaelt zu jedem Spektrum der Datenbasis die Omega-,Thetapos.
    //! und einen Reportwert (integrale Intensitaet)
    LPCurve ScanReport;

    //! Kurve enthaelt weitere Zusatzinformationen zu den Kurven der Datenbasis
    //! Absorberposition, Monitorintensitaet, reale Messzeit
    LPCurve ScanReport2;


    //! abhaengig von eSaveFormat
    //! Rueckkehrcode bei ShiftedStandard ist die Anzahl der Psd-kanaele die
    //! benoetigt werden um den Thetabereich
    //! zu ueberdecken
    //! sonst immer 0
    int GetAdditionalColumns( void );

    //! Funktion liefert zu Anzahl der Scans (1.Parameter) Anzahl der
    //! Psd-kanaele, die benoetigt werden um den Thetabereich zu ueberdecken
    //! (nur bei ShiftedStandard)
    //! sonst 0 als Rueckkehrcode
    int GetShift( int ScanNumber );

    //! soll AreaScanstart vor Aufruf einer Einstellungsbox verhindern
    //! FIX Fehler 2
    BOOL bScanSetupOk;

	 //! Aenderung DB_Zerlegung
	 BOOL DismantleDB( void );
	 BOOL ComposeDB( void );
	 BOOL SaveDismantleCurve( int );
	 BOOL bComposeDB;
};

class TAreaScanCCDParameters
{

  public:

         TAreaScanCCDParameters( void );

         //! enthaelt Nummer des Omegamotors in der Motorenliste
         int nOmega;

         //! enthaelt Anfangs- und Endwinkel sowie die Schrittweite des Omegamotors
         //! fuer einen Areascan
			double dOmegaMin, dOmegaMax, dOmegaWidth;

         //! gibt an in welcher Form die Messwerte in Datei abgelegt werden sollen
			//! (nur StandardFile verwendet)
         TSaveFormat eSaveFormat;

         //! Speichern nach Beendigung der Messung
         BOOL bSaveOnReady;

         //! gibt an ob ein CCD angeschlossen
         BOOL bCCDDevice;

         //! gibt ausgewaehlten Detektor an
         LPDevice Device;

         //! enthaelt die maximale Messzeit
         float fMaxTime;

         //! enthaelt die Totzeit
         float fDelay;

         //! enthlt Hardwaremodus
         int iHardwareMode;

         //! setzt den Detektor, dessen id-nummer dem Param. id entspricht
         //! dabei wird bPsdDevice gesetzt, wenn der Detektor ein Psd ist
         void SetDevice( int );

};

class TAreaScanCCD : public TAreaScanCCDParameters, public TPlotData //, public TMDIWindow
{

  public:

         TAreaScanCCD( void );
         ~TAreaScanCCD( void );

  protected:

         //! loescht aktuelle datenbasis + baut AreaScanfenster neu auf
         //! Rueckkehrcode immer True
         BOOL New( void );

         //! Darstellen des AreaScanCCD-Fenster
         void Create( HWND );

         //! aufruf wenn rechte Maustaste gedrueckt
         //! Fenstermenue an aktueller Mausposition ausgeben
         //! eingabeparam. aktuelle Mauspos.low(8) x-pos
         //!                                high(8) y-pos.
         void rButtonDown( LPARAM );

         //! behandelt einige Menuekommandos
         void InitializeDlg( WPARAM, LPARAM );

         //! wird aufgerufen aus m_main.cpp bei cm_InitializeScan
         //! startet den Scanvorgang
			void InitializeTask( WPARAM, LPARAM );

  private:

         //! gibt Nummer der Kurve in Datenbasis an, die gerade aktiv ist
         int nActiveScan;

         //! setzt Titel des AreaScanCCD-Fensters
         //! Rueckkehrcode immer True
         BOOL SetTitle( void );

};


#endif   
// __DIFRKMTY_H
// __LastLine__