// (C) 1993, 1994 by Heiko Damerow MPG AG "Roentgenbeugung"
#ifndef __M_MOTCOM_H
#define __M_MOTCOM_H

typedef enum
{
  CorrLinear = 1, CorrPolynom
} TCorrect;

typedef enum
{
  ToAngle = 1, ToPosition
} TTrans;

typedef struct
{
  double   dAngleMin, dAngleMax, dAngle, dSpeed, dAngleWidth;
}
TMSettings;

class __rtti TMotor
{
    friend class   TMList;
    friend class   TCalibrate;
    friend class   TPositionControl;
    friend class   TMotorParam;

  public:
    TMotor( void );
    virtual        ~TMotor( void ) {};


    LPCSTR         pCharacteristic( void ) { return (LPCSTR)szCharacteristic; };
    LPCSTR         pSF( void )   { return (LPCSTR)SFmt; };
    LPCSTR         pDF( void )   { return (LPCSTR)DFmt; };
    int            GetId( void )   { return nId; };
    BOOL           IsRangeHit( void )   { return bRangeHit; };
    BOOL           SetAngleRange( void );
    BOOL           GetAngle( BOOL );
    BOOL           MoveToAngle( double );
    BOOL           MoveByAngle( double );
    BOOL           PushSettings( void );
    BOOL           PopSettings( TMParameter );
    BOOL           SetAngleWidth( double );
    void           SetRelativeZero( void )   {};
    void           ResetRelativeZero( void )  {};
    double         GetAngleWidth( void )  { return dAngleWidth; };
    DWORD          GetLimit( void )  { return dwRemoveLimit; };
    virtual void   OptimizingDlg( void )   { return; };
    virtual void   StartCheckScan( void )   { return; };
    virtual BOOL   SetLimit( DWORD )   { return TRUE; };
    virtual BOOL   StopLimitWatch( void )  { return R_OK; };
    virtual BOOL   StartLimitWatch( void )  { return R_OK; };
    virtual int    ExecuteCmd( LPSTR )  { return R_OK; };
    virtual BOOL   ActivateDrive( void )  { return TRUE; };
    virtual BOOL   IsMoveFinish( void )  { return TRUE; };
    virtual BOOL   SetLine( int, BOOL )  { return TRUE; };
    virtual BOOL   IsLimitHit( void )  { return TRUE; };
    virtual BOOL   StartToIndex( long & )  { return TRUE; };
    virtual BOOL   IsIndexArrived( void )  { return TRUE; };
    virtual BOOL   StopDrive( int )      { return TRUE; };
    virtual BOOL   GetPosition( BOOL )  { return TRUE; };
    virtual long   GetFailure( void )    { return 0; };
    virtual BOOL   MoveByPosition( long );
    virtual BOOL   MoveToPosition( long );
    virtual BOOL   SetHome( void );

    virtual BOOL   SetMoment( float )   { return TRUE; };
    virtual float  GetMoment( void )  { return 1.0; };

    // Speed: Geschwindigkeit in Nutzereinheiten
    virtual BOOL   SetSpeed( double )  { return TRUE; };
    virtual double GetSpeed( void )   { return 1.0; };
    double         GetMaxSpeed() { return (double)(dwMaxVelocity / fSpeedScale); }

    // Beschleunigung in Nutzereinheiten
    double         GetAcceleration() {
                      double a;
                      Translate(a, dwAcceleration);
                      return fabsl(a);
                   }
    // minimale Schrittweite in Nutzereinheiten
    double         GetMinAngleWidth() {
                      double a;
                      if( eUnit==Grad)
                         a = fabs( dKoeff_1 * wPositionMinWidth / 3600.0 );
                      else
                        if(eUnit==Minuten)
                          a = fabs( dKoeff_1 * wPositionMinWidth / 60.0 );
                        else
                          a = fabs( dKoeff_1 * wPositionMinWidth );
                      return a;
                   }

    long           PreviousPosition( void )  { return lPreviousPosition; };
    void           SetCorrectionState( BOOL );
    virtual int    SaveSettings( BOOL );
    virtual int    Initialize( void );
    virtual WORD   GetStatus( void )    { return 0xFF; };

    WORD           wPositionMinWidth;
    WORD           wPositionMaxWidth;
    long           lPreviousPosition;
    long           lDeltaPosition;
    long           lPositionMax;
    long           lPositionMin;
    WORD           wPositionWidth;
    long           lPosition;
    double         dAngle;
    char           szUnit[ 21 ];
    TUnitType      eUnit;
    double         dAngleBias;
    double         dAngleWidth;
    double         dAngleMin, dAngleMax;
    BOOL           IsCalibrated( void )    { return bIsCalibrated; };

  protected:
    void           SetCalibrationState( BOOL state )  { bIsCalibrated = state; };
    double         dSpeed;
    BOOL           bUpwards;
    int            nId;
    BOOL           bInquireStatus;
    BOOL           bRangeHit;
    BOOL           bIndexLine;
    BOOL           bMoveFirstToLimit;
    BOOL           bIndexDetected;
    DWORD          dwRemoveLimit;
    double         dInitialAngle;
    long           lDistanceToZero;
    BOOL           bControlBoardOk;
    DWORD          dwHysteresis;
    float          fSpeedScale;
    long           lCorrPos;
    BOOL           bLimitWatchActive;
    DWORD          dwVelocity;
    DWORD          dwMaxVelocity;
    DWORD          dwAcceleration;
    char           SFmt[ 6 ];
    char           DFmt[ 6 ];

  private:
    BOOL           Translate( long&, double ); // angle to position
    BOOL           Translate( double&, long ); // position to angle
    void           funcd( double arg, double offset, double* f, double* df );
    int            rtsave( double offset, double x1, double x2, double xacc,
                           double* ret );

    BOOL           bIsCalibrated;
    BOOL           bInitialMoveReady;
    static int     nMaxId;
    double         dKoeff_3, dKoeff_2, dKoeff_1;
    DWORD          dwInterval;
    BOOL           bCorrection;
    float          fCorrBias;
    TCorrect       eCorrStatus;
    char           szCharacteristic[ MaxString+1 ];
    TMSettings     Settings;
};

class  __rtti TMList
{
  public:
    TMList( int, HINSTANCE );
    ~TMList( void );

    int       AD_Z, AD_Y, AD_X;
    int       AD_Tilt, AD_DC, AD_Collimator, AD_DF;
    int       AD_Encoder;
    int       AD_Phi;
    int       AD_Psi;
    int       AD_Rotation;
    int       AD_Absorber, AD_Monochromator;
    int       AD_Theta;
    int       AD_Omega;
    TMotor*   MP( void );
    TMotor*   MP( int );
    void      SetAngleDefault( void );
    void      PositionControlDlg( void );
    void      SetParametersDlg( void );
    void      InquireReferencePointDlg( int );
    BOOL      SetAxis( int );
    int       GetAxis( void )  { return nActiveDrive; };
    int       LastId( void )   { return nLastDrive; };
    HINSTANCE GetModuleHandle( void )    { return hMInstance; };
    BOOL      InitializeModule( void );
    BOOL      SaveModuleSettings( void );
    TAxisType ParsingAxis( LPSTR );
    int       GetAxisNumber( void )  { return ( nLastDrive+1 ); };

  private:
    HINSTANCE hMInstance;
    int       nMaxNumber;
    int       nLastDrive;   // points the last axis
    int       nActiveDrive; // points the actual axis
    TMotor**  aMotor;
};

typedef TMList FAR * LPMList;
typedef TMotor FAR * LPMotor;
#endif
// __LastLine__

